<?php
/*
Plugin Name: Extended Page Lists
Plugin URI: http://wordpress.grandslambert.com/plugins/extended-page-lists.html
Description: Add custom configured page lists to your posts, pages and sidebars.
Version: 0.6
Author: GrandSlambert
Author URI: http://wordpress.grandslambert.com/

**************************************************************************

Copyright (C) 2009 GrandSlambert

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************

*/

class ExtendedPageLists {
    /* Plugin Settings */
    var $version        = '0.6';
    var $optionsName    = "extended-page-lists-options";
    var $menuName	= "extended-page-lists-options";
    var $pluginName     = 'Extended Page Lists';
    var $options        = array();

    /**
     * Class Constructor
     */
    function ExtendedPageLists() {
        $this->pluginPath = WP_CONTENT_DIR . "/plugins/" . basename(dirname(__FILE__));
        $this->loadSettings();

        /* Add Shortcodes */
        add_shortcode('epl', array($this, 'listPages') );

        /* Add Options Pages and Links */
        add_action('admin_menu', array(&$this, "addAdminPages"));
        add_filter('plugin_action_links', array(&$this, "addConfigureLink"), 10, 2);
        add_action('admin_init', array(&$this, 'registerOptions'));
        add_filter('whitelist_options', array(&$this, 'whitelistOptions'));
    }

    /**
     * Load plugin settings
     */
    function loadSettings() {
        $this->options = get_option($this->optionsName);

        /* Get non-false defaults */
        if (!$this->options['excerpt-length'])
            $this->options['excerpt-length'] = 55;

        if (!$this->options['excerpt-more'])
            $this->options['excerpt-more'] = '...';
    }

    /**
     * Add all options to the whitelist for the NONCE
     * Required for Wordpress MU support
     */
    function whitelistOptions($whitelist) {
        if (is_array($whitelist)) {
            $option_array = array($this->pluginName => $this->optionsName);
            $whitelist = array_merge($whitelist, $option_array);
        }

        return $whitelist;
    }

    /**
     * Outputs the options sub panel
     */
    function optionsPanel() {
    // Include options panel
        include($this->pluginPath . "/options-panel.php");
    }

    /**
     * Adds Disclaimer options tab to admin menu
     */
    function addAdminPages() {
        global $wp_version;

        add_options_page($this->pluginName, $this->pluginName, 8, $this->menuName, array(&$this, 'optionsPanel'));

        // Use the bundled jquery library if we are running WP 2.5 or above
        if (version_compare($wp_version, "2.5", ">=")) {
            wp_enqueue_script("jquery", false, false, "1.2.3");
        }
    }

    /**
     * Adds a settings link next to Login Configurator on the plugins page
     */
    function addConfigureLink($links, $file) {
        static $this_plugin;

        if (!$this_plugin) {
            $this_plugin = plugin_basename(__FILE__);
        }

        if ($file == $this_plugin) {
            $settings_link = '<a href="' . get_option('siteurl') . '/wp-admin/options-general.php?page=' . $this->menuName . '">' . __('Settings') . '</a>';
            array_unshift($links, $settings_link);
        }

        return $links;
    }


    /**
     * List Pages Short Code
     */
    function listPages($atts) {
        global $post;

        extract(shortcode_atts(array(
            // Page Query Settings
            'child_of' 	=> $post->ID,
            'sort_order' 	=> 'ASC',
            'sort_column'	=> 'post_title',
            'hierarchical'	=> 1,
            'exclude' 	=> NULL,
            'include' 	=> NULL,
            'meta_key' 	=> NULL,
            'meta_value' 	=> NULL,
            'authors' 	=> NULL,
            'parent' 	=> -1,
            'exclude_tree'  => NULL,
            'number' 	=> NULL,
            'offset' 	=> 0,

            // Display Settings
            'blocktag'	=> 'ul',
            'blockclass'	=> 'page-list',
            'itemtag'	=> 'li',
            'itemclass'	=> 'page-list-item',
            'titletag'	=> NULL,
            'titleclass'	=> 'page-list-title',
            'target'	=> $this->options['target'],
            'showcontent'	=> false,
            'contenttag'	=> 'p',
            'contentclass'	=> 'page-list-content',
            'showexcerpt'	=> false,
            'excerpttag'	=> 'p',
            'excerptclass'	=> 'page-list-excerpt',
            'excerptlength' => $this->options['excerpt-length'],
            'excerptmore'	=> $this->options['excerpt-more'],
            'beforelink'	=> NULL,
            'morelink'	=> $this->this->options['more-link'],
            ), $atts));


        if ($exclude)
            $params.= '&exclude=' . $exclude;

        if ($include)
            $params.= '&include=' . $include;

        if ($meta_key)
            $params.= '&meta_key=' . $meta_key;

        if ($meta_value)
            $params.= '&meta_value=' . $meta_value;

        if ($authors)
            $params.= '&authors=' . $authors;

        if ($exclude_tree)
            $params.= '&exclude_tree=' . $exclude_tree;

        if ($number)
            $params.= '&number=' . $number;

        if ($offset)
            $params.= '&offset=' . $offset;

        // Get the pages
        $pages = get_pages('child_of=' . $child_of . '&parent=' . $parent . '&sort_column='. $sort_column . '&sort_order=' . $sort_order . $params);

        // Display the pages
        if ($target) {
            $target = ' target="' . $target . '"';
        } else {
            unset ($target);
        }

        if ($blocktag) {
            $output.= '<' . $blocktag . ' class="' . $blockclass . '">';
        }

        foreach ($pages as $page) {
            $pageLink = get_page_link($page->ID);

            if ($itemtag)
                $output.= '<' . $itemtag . ' id="' . $itemclass . '-' . $page->ID . '" class="' . $itemclass . ' ' . $itemclass . '-' . $page->ID . '">';

            if ($titletag)
                $output.= '<' . $titletag . ' class="' . $titleclass . ' ' . $titleclass . '-' . $page->ID .  '">';

            $output.= '<a href="' . $pageLink . '" ' . $target . '>' . $page->post_title . '</a>';

            if ($titletag) {
                $output.= '</' . $titletag . '';
            }

            if ($showcontent) {
                if ($contenttag) {
                    $output.= '<' . $contenttag . ' class="' . $contentclass . '">';
                }

                $content = $page->post_content;

                $output.= apply_filters('the_content', $content);

                if ($contenttag)
                    $output.= '</' . $contenttag . '">';
            }


            if ($showexcerpt) {
                if ($morelink) {
                    $excerptlink = $beforelink . '<a href="' . $pageLink . '" ' . $target . '>' . $excerptmore .'</a>';
                } else {
                    $excerptlink = $excerptmore;
                }

                if ($excerpttag) {
                    $output.= '<' . $excerpttag . ' class="' . $excerptclass . '">';
                }

                if (!$excerpt = get_post_meta($page->ID, 'epl-text', true) ) {
                    $excerpt = $this->trimExcerpt($page->post_content, $excerptlength, $excerptlink);
                }

                $output.= apply_filters('the_excerpt', $excerpt);

                if ($excerpttag) {
                    $output.= '</' . $excerpttag . '">';
                }
            }

            if ($itemtag) {
                $output.= '</' . $itemtag . '>';
            }
        }

        if ($blocktag) {
            $output.= '</' . $blocktag . '';
        }

        return $output;
    }

    /**
     * Trim the excerpt
     */
    function trimExcerpt($text, $length = NULL, $more = NULL) {
        global $post;

        if (!$length) {
            $length = $this->options['excerpt-length'];
        }

        if (!$more) {
            $more = $this->options['excerpt-more'];
        }

        $text = apply_filters('the_content', $text);
        $text = str_replace(']]>', ']]&gt;', $text);
        $text = strip_tags($text, '<p>');
        $text = preg_replace('@<script[^>]*?>.*?</script>@si', '', $text);
        $words = explode(' ', $text, $length + 1);
        
        if (count($words)> $length) {
            array_pop($words);
            array_push($words, $more);
            $text = implode(' ', $words);
        }

        return $text;
    }

    /**
     * Show the version number
     */
    function showVersion() {
        return $this->version;
    }

    /**
     * Register the options for Wordpress MU Support
     */
    function registerOptions() {
        register_setting( $this->optionsName, $this->optionsName);
    }

    /**
     * Display the list of contributors.
     * @return boolean
     */
    function contributorList() {
        $this->showFields = array('NAME', 'LOCATION' , 'COUNTRY');
        print '<ul>';

        $xml_parser = xml_parser_create();
        xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
        xml_set_element_handler($xml_parser, array($this,"startElement"), array($this, "endElement") );
        xml_set_character_data_handler($xml_parser, array($this, "characterData") );

        if (!(@$fp = fopen('http://wordpress.grandslambert.com/xml/extended-page-lists/contributors.xml', "r"))) {
            print 'There was an error getting the list. Try again later.';
            return;
        }

        while ($data = fread($fp, 4096)) {
            if (!xml_parse($xml_parser, $data, feof($fp))) {
                die(sprintf("XML error: %s at line %d",
                    xml_error_string(xml_get_error_code($xml_parser)),
                    xml_get_current_line_number($xml_parser)));
            }
        }

        xml_parser_free($xml_parser);
        print '</ul>';
    }

    /**
     * XML Start Element Procedure.
     */
    function startElement($parser, $name, $attrs) {
        if ($name == 'NAME') {
            print '<li class="bbw-contributor">';
        } elseif ($name == 'ITEM') {
            print '<br><span class="bbw-contributor_notes">Contributed: ';
        }

        if ($name == 'URL') {
            $this->makeLink = true;
        }
    }

    /**
     * XML End Element Procedure.
     */
    function endElement($parser, $name) {
        if ($name == 'ITEM') {
            print '</li>';
        }
        elseif ($name == 'ITEM') {
            print '</span>';
        }
        elseif ( in_array($name, $this->showFields)) {
            print ', ';
        }
    }

    /**
     * XML Character Data Procedure.
     */
    function characterData($parser, $data) {
        if ($this->makeLink) {
            print '<a href="http://' . $data . '" target="_blank">' . $data . '</a>';
            $this->makeLink = false;
        } else {
            print $data;
        }
    }

    function activate() {

        /* Compile old options into new options Array */
        $options = array(
            'epl_default_target'    => 'target',
            'epl_widget_target'     => 'widget-target',
            'epl_custom_css'        => 'custom-css',
            'epl_excerpt_length'    => 'excerpt-length',
            'epl_excerpt_more'      => 'excerpt-more',
            'epl_excerpt_more_link' => 'more-link'
        );

        foreach ($options as $old=>$new) {
            if ($old_option = get_option($old)) {
                $this->options[$new] = $old_option;
                delete_option($old);
            }
        }

        if (!add_option($this->optionsName, $this->options)) {
            update_option($this->optionsName, $this->options);
        }
    }
}

// Pre 2.6 Compatibility
if ( !defined('WP_CONTENT_DIR') )
    define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );

$EPLOBJ = new ExtendedPageLists;
require_once($EPLOBJ->pluginPath . '/widget.php');
register_activation_hook(__FILE__, array($EPLOBJ, 'activate') );
